/*	DOS Stamp Communications functions
	(C)1999 Bagotronix, Inc.
	Author: Ivan Baggett

	Written in Borland C v3.0.
	For Internal UARTs of AM188ES

	These functions constitute the basic requirements for most serial
	communications.  The exotic features of the UARTs are not explored
	here.  For most data transfer and control applications, these
	functions are sufficient.

	Some of the exotic UART features are DMA transfers, 9-bit protocols,
	and hardware flow control.  Read the AM188ES User's Manual for further
	information about these.
*/

#include <dos.h>

#include "stdinc.h"
#include "dosstamp.h"
#include "dscomm.h"


/*	Sets baud rate
		arguments:
			1) which port (0 or 1)
			2) the baud rate divisor (must be nonzero)
		comments:
			Baud rate divisor may have to be changed if the power save
			modes are activated.
			Improper operation will result from out of range arguments.
*/
void SERsetbaud (UBYTE which, UWORD bauddiv) {
	outport (which ? SER1_BDIV : SER0_BDIV, bauddiv);
}


/*	Checks if a character has been received
		arguments:
			1) which port (0 or 1)
		returns:
			0 if no character available (RDR == 0)
			nonzero if character available (RDR == 1)
		comments:
			Improper operation will result from out of range arguments.
*/
BOOLEAN SERrcvrready (UBYTE which) {
	return (inport (which ? SER1_STS : SER0_STS) & 0x0080);
}


/*	Checks if it is OK to write to transmitter
		arguments:
			1) which port (0 or 1)
		returns:
			0 if not OK (THRE == 0)
			nonzero if OK (THRE == 1)
		comments:
			Improper operation will result from out of range arguments.
*/
BOOLEAN SERxmtrready (UBYTE which) {
	return (inport (which ? SER1_STS : SER0_STS) & 0x0040);
}


/*	Checks if transmitter is done
		arguments:
			1) which port (0 or 1)
		returns:
			0 if not done (TEMT == 0)
			nonzero if done (TEMT == 1)
		comments:
			Improper operation will result from out of range arguments.
*/
BOOLEAN SERxmtrdone (UBYTE which) {
	return (inport (which ? SER1_STS : SER0_STS) & 0x0004);
}


/*	Get a character from the serial port
		arguments:
			1) which port (0 or 1)
		returns:  the character
		comments:  assumes there is a character available.
			Check to see if a character is available first with
			SERrcvrready().
			Improper operation will result from out of range arguments.
*/
BYTE SERgetc (UBYTE which) {
	return (inport (which ? SER1_RDATA : SER0_RDATA));
}


/*	Put a character to the serial port
		arguments:
			1) which port (0 or 1)
			2) the character to be sent
		returns:  the character to be sent
		comments:  assumes the transmitter is available.
			Check to see if transmitter is available first with
			SERxmtrready().
			Improper operation will result from out of range arguments.
*/
BYTE SERputc (UBYTE which, BYTE b) {
	outport (which ? SER1_TDATA : SER0_TDATA, b);
	return (b);
}


/*	Check serial port status
		arguments:
			1) which port (0 or 1)
		returns:  contents of the status register
			Improper operation will result from out of range arguments.
*/
UWORD SERstatus (UBYTE which) {
	return (inport (which ? SER1_STS : SER0_STS));
}


/*	Initialize serial port for 8 data, no parity, 1 stop bits
		arguments:
			1) which port (0 or 1)
			2) interrupt on receive (0 = disabled, 1 = enabled)
			3) interrupt on transmit (0 = disabled, 1 = enabled)
			4) interrupt on error (0 = disabled, 1 = enabled)
		comments:
			Does not enable the interrupt controller or set the interrupt
			vector.  Also, PIO pins must be configured for "normal"
			operation to get the serial function.
			Improper operation will result from out of range arguments.
*/
void SERinit (UBYTE which, BOOLEAN rif, BOOLEAN tif, BOOLEAN eif) {
	UWORD u;

	// set bits from flags
	u = (rif ? 0x0080 : 0) | (tif ? 0x0100 : 0) | (eif ? 0x1000 : 0);
	u |= 0x0061;	// enabled transmitter and receiver, mode 1
	outport (which ? SER1_CON : SER0_CON, u);
}
